import { NgModule, APP_INITIALIZER } from '@angular/core';

import { BrowserModule, Title } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

import { HttpClientModule } from '@angular/common/http';

import { AppRoutingModule } from './app-routing.module';
import { AppStoreModule } from './app-store.module';
import { MaterialModule } from './commons/ui/material/material.module';
import { CovalentModule } from './commons/ui/covalent/covalent.module';

import { AppComponent } from './app.component';

import { environment } from '../environments/environment';
import { RESTService } from 'packages/commons/service/rest.service';
import { RPCService } from 'packages/commons/service/rpc.service';

import {
  L10nConfig,
  L10nLoader,
  LocalizationModule,
  LocaleValidationModule,
  StorageStrategy,
  ProviderType
} from 'angular-l10n';

const l10nConfig: L10nConfig = {
  locale: {
      languages: [
          { code: 'en', dir: 'ltr' },
          { code: 'kr', dir: 'ltr' },
      ],
      defaultLocale: { languageCode: 'en', countryCode: 'US' },
      storage: StorageStrategy.Cookie,
      cookieExpiration: 30
  },
  translation: {
      providers: [
          { type: ProviderType.Static, prefix: './assets/translations/of-' }
      ],
      caching: true,
      composedKeySeparator: '.',
      missingValue: 'No key',
      i18nPlural: true
  }
};

export function initL10n(l10nLoader: L10nLoader): Function {
  return () => l10nLoader.load();
}

@NgModule({
  declarations: [
    AppComponent,
  ],
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
    AppRoutingModule,
    AppStoreModule,
    MaterialModule,
    CovalentModule,
    HttpClientModule,
    LocalizationModule.forRoot(l10nConfig),
    LocaleValidationModule.forRoot()
  ],
  providers: [
    {provide: 'REST_BASE_URL', useValue: environment.restBaseURL},
    {provide: 'RPC_BASE_URL', useValue: environment.rpcBaseURL},
    RESTService, RPCService,
    Title,
    {
      provide: APP_INITIALIZER,
      useFactory: initL10n,
      deps: [L10nLoader],
      multi: true
    }
  ],
  bootstrap: [AppComponent]
})
export class AppModule { }
